#!/usr/bin/env node


'use strict';

var path = require('path');
var fs = require('fs');
var febs = require('febs');
var colors = require('colors');
var prompt = require('prompt');
var argv = require('minimist')(process.argv.slice(2));

var spawn = require('child_process').spawn;

colors.setTheme({  
    silly: 'rainbow',  
    input: 'grey',  
    verbose: 'cyan',  
    prompt: 'red',  
    info: 'green',  
    data: 'blue',  
    help: 'cyan',  
    warn: 'yellow',  
    debug: 'magenta',  
    error: 'red'  
});


/**
* @desc: 
* @return: 
*/
function installVueConfirmation(cb) {
  prompt.start();

  var property = {
    name: 'rncli',
    message: 'Did you already install ' + 'vue-cli'.info + '?',
    validator: /y[es]*|n[o]?/,
    warning: 'Must respond yes or no',
    default: 'yes'
  };

  prompt.get(property, function (err, result) {
    if (result.rncli[0] === 'n') {
      console.log('\r\n');
      console.log('run `npm i vue-cli -g` first');
      process.exit(1);
    } else {
      cb && cb();
    }
  });
}


function installNuxt(projectName, cb) { 
  // react-native.
  console.log('It will take a while...'.input);
  console.log('Creating a ' + 'nuxt'.info + ' project...');
  
  var proc = spawn('vue', [
    'init',
    'nuxt-community/starter-template',
    projectName
  ], {stdio: 'inherit'});

  proc.on('close', function (code) {
    if (code !== 0) {
      console.error('install nuxt template' + ' failed');
      console.error(code);
      process.exit(1);
    } else {
      cb&&cb(code);
    }
  });
}


function installFebsvue(root, projectName) {

  var dirTemplate = path.join(__dirname, '..', 'template');

  var copyFile = febs.utils.denodeify(febs.file.fileCopy);
  var copyDir = febs.utils.denodeify(febs.file.dirCopy);

  // backup.
  var now = new Date();
  now = now.toString();
  febs.file.dirAssure(path.join(root, '_backup', now));

  var copyAppHtml = febs.file.fileIsExist(path.join(root, 'app.html')) 
                  ? copyFile(path.join(root, 'app.html'), path.join(root, '_backup', now, 'app.html'))
                  : new Promise(function(resolve){resolve();}); 

  Promise.all([
    copyFile(path.join(root, 'nuxt.config.js'), path.join(root, '_backup', now, 'nuxt.config.js')),,
    copyFile(path.join(root, 'package.json'), path.join(root, '_backup', now, 'package.json')),
    copyDir(path.join(root, 'layouts'), path.join(root, '_backup', now, 'layouts')),
    copyDir(path.join(root, 'plugins'), path.join(root, '_backup', now, 'plugins')),
    // copyDir(path.join(root, 'pages'), path.join(root, '_backup', now, 'pages')),
    copyAppHtml
  ])
  .then(function(){
    febs.file.fileRemove(path.join(root, 'package.json'));
    febs.file.fileRemove(path.join(root, 'nuxt.config.js'));
    febs.file.fileRemove(path.join(root, 'nuxt.config.static.js'));
    febs.file.fileRemove(path.join(root, 'app.html'));
    febs.file.dirRemoveRecursive(path.join(root, 'layouts'));

    if (febs.file.fileIsExist(path.join(root, 'plugins', 'febsvue.js'))) {
      febs.file.fileRemove(path.join(root, 'plugins', 'febsvue.js'));
    }
    // febs.file.dirRemoveRecursive(path.join(root, 'pages'));
  })
  .then(function(){
    return Promise.all([
      copyFile(path.join(dirTemplate, 'nuxt.config.static.js'), path.join(root, 'nuxt.config.static.js')),
      copyFile(path.join(dirTemplate, 'nuxt.config.js'), path.join(root, 'nuxt.config.js')),
      copyFile(path.join(dirTemplate, 'package.json'), path.join(root, 'package.json')),
      copyFile(path.join(dirTemplate, 'app.html'), path.join(root, 'app.html')),
      copyDir(path.join(dirTemplate, 'layouts'), path.join(root, 'layouts')),
      copyFile(path.join(dirTemplate, 'plugins', 'febsvue.js'), path.join(root, 'plugins', 'febsvue.js')),
      // copyDir(path.join(dirTemplate, 'pages'), path.join(root, 'pages')),
    ])
  })
  .then(function(){
    var proc = spawn('npm', ['install'], {stdio: 'inherit'});
    proc.on('close', function (code) {
      if (code !== 0) {
        console.error('`npm install` failed');
        return;
      } else {
        console.log('');
        console.log('*  copy pages & layouts to _backup/'+now);
        console.log('');
        console.log('************************');
        console.log('*  # serve with hot reload at localhost:3000');
        console.log('*  \'npm run dev\' .  *');
        console.log('');
        console.log('*  # generate static project');
        console.log('*  \'npm run static\' .  *');
        console.log('************************');
        console.log('');
        console.log('');        
        process.exit(0);
      }
    });
  })
  .catch(function(err){
    console.log(err);
    process.exit(0);
  });
}

/**
* @desc: 
* @return: 
*/
function createProject(name) {
  var root = path.resolve(name);
  var projectName = path.basename(root);

  installVueConfirmation(function(){
    installNuxt(projectName, function(){
      if (!fs.existsSync(root)) {
        fs.mkdirSync(root);
      }

      process.chdir(root);

      installFebsvue(root, projectName);
    });    
  });
}

/**
* @desc: 
* @return: 
*/
function createAfterConfirmation(name) {
  prompt.start();

  var property = {
    name: 'yesno',
    message: 'Directory ' + name + ' already exists. Continue?',
    validator: /y[es]*|n[o]?/,
    warning: 'Must respond yes or no',
    default: 'no'
  };

  prompt.get(property, function (err, result) {
    if (result.yesno[0] === 'y') {
      createProject(name);
    } else {
      console.log('Project initialization canceled');
      process.exit();
    }
  });
}

/**
* @desc: 
* @return: 
*/
function validatePackageName(name) {
  if (!name.match(/^[$A-Z_][0-9A-Z_$]*$/i)) {
    console.error(
      '"%s" is not a valid name for a project. Please use a valid identifier ' +
        'name (alphanumeric).',
      name
    );
    process.exit(1);
  }

  if (name.toLowerCase() === 'vue') {
    console.error(
      '"%s" is not a valid name for a project. Please do not use the ' +
        'reserved word "Vue".',
      name
    );
    process.exit(1);
  }
}

/**
* @desc: 
* @return: 
*/
function init(name) {
  validatePackageName(name);

  if (fs.existsSync(name)) {
    createAfterConfirmation(name);
  } else {
    createProject(name);
  }
}


// minimist api
var commands = argv._;
if (commands.length === 0) {
  console.error(
    'You did not pass any commands, did you mean to run `' + 'febs-vue init'.info + '`?'
  );
  process.exit(1);
}

switch (commands[0]) {
case 'init':
  if (!commands[1]) {
    console.error(
      'Usage: ' + 'febs-vue init <ProjectName>'.info
    );
    process.exit(1);
  } else {
    init(commands[1]);
  }
  break;
default:
  console.error(
    'Command `%s` unrecognized. ' +
    'Make sure that you have run `' + 'npm install'.info + '` and that you are inside a febs-vue project.',
    commands[0]
  );
  process.exit(1);
  break;
}