
import VueRouter from "vue-router";

export namespace api {

  namespace dialog {
    /**
     * @desc: 隐藏对话框.
     * @param dialogObj: 隐藏指定的窗口; 如果不提供, 则隐藏所有窗口.
     */
    function hide(dialogObj?:any): void;
    /**
     * @desc: 显示警告对话框. (回调函数的上下文为当前窗口)
     * @param cfg: {
     * blackBg: 是否使用黑色背景.
     * cssClass: 自定义样式.
     * title:    标题.
     * content:	内容文字.
     * contentHtml: html格式的内容 (与content二选一)
     * confirm: function(dialog:any){}	// 点击确认键的回调.
     * okText
     * }
     *  or string.
     * @return 返回此窗口的句柄.
     */
    function alert(cfg: string | {
      blackBg?: boolean,
      cssClass?: string,
      title?: string,
      content?: string,
      contentHtml?: string,
      confirm?: (dialog:any) => void,
      okText?: string,
    }): any;
    /**
     * @desc: 显示确认对话框. (回调函数的上下文为当前窗口)
     * @param cfg: {
     * blackBg: 是否使用黑色背景.
     * cssClass: 自定义样式.
     * title:    标题.
     * content:	内容文字.
     * contentHtml: html格式的内容 (与content二选一)
     * confirm: function(dialog:any){}	// 点击确认键的回调.
     * cancel: function(dialog:any){}	// 点击取消键的回调.
     * okText 确认按钮文字
     * cancelText: 取消按钮文字
     * }
     * @return 返回此窗口的句柄.
     */
    function confirm(cfg: {
      blackBg?: boolean,
      cssClass?: string,
      title?: string,
      content?: string,
      contentHtml?: string,
      confirm?: (dialog:any) => void,
      cancel?: (dialog:any) => void,
      okText?: string,
      cancelText?: string,
    }): any;
    /**
     * @desc: 显示文本输入确认对话框. (回调函数的上下文为当前窗口)
     * @param cfg: {
     * blackBg: 是否使用黑色背景.
     * cssClass: 自定义样式.
     * title:    标题.
     * content:		 内容文字.
     * contentHtml: html格式的内容 (与content二选一)
     * editText:		 输入框文字.
     * confirm: function(text, dialog:any){}	// 点击确认键的回调.
     * cancel:  function(dialog:any){} // 点击取消键的回调.
     * okText:
     * cancelText:
     * }
     * @return 返回此窗口的句柄.
     */
    function confirmEdit(cfg: {
      blackBg?: boolean,
      cssClass?: string,
      title?: string,
      content?: string,
      contentHtml?: string,
      editText?: string,
      confirm?: (text:string, dialog:any) => void,
      cancel?: (dialog:any) => void,
      okText?: string,
      cancelText?: string,
    }): any;
  }

  namespace loading {
    /**
     * @desc: 当前是否有loading显示.
     */
    function isVisible(): boolean;
    /**
     * @desc: 使用延时显示加载框.
     * @param content: 提示文本.
     * @param timeout: 延时显示, 默认为0.
     * @param spinClass: 默认为 febsui-icon-spin1-white
     * @param spinLeft: 是否在左侧显示spin.
     * @param blackBg: 是否使用黑色背景. (默认黑色)
     * @return: 
     */
    function show(cfg: string | {
      content?: string,
      timeout?: number,
      spinClass?: string,
      spinLeft?: boolean,
      blackBg?: boolean,
    }): void;
    /**
     * @desc 隐藏.
     */
    function hide(): void;
  }

  namespace toast {
    /**
   * @desc: 显示提示.
   * @param cfg: {
   * cfg.content:    标题.
   * cfg.durable:	持续的时间 ms.
   * cfg.icon: 前置图标.  // "ok" "warn" "error" 默认null, 没有图标
   * cfg.callback: function(){}	// 对话框消失后的回调.
   * cfg.center: 默认为false; 是否使用居中的显示方式.
   * }
   *  or string.
   */
    function show(cfg: string | {
      content?: string,
      durable?: number,
      icon?: 'ok' | 'warn' | 'error' | null | string,
      callback?: () => void,
      center?: boolean,
    }): void;
    /**
     * @desc 手动隐藏toast.
     */
    function hide(): void;
  }
}

export namespace utils {
  namespace dom {

    /**
    * @desc: 判断指定的地址是否是外部网站的地址.
    */
    function isExternPath(path: string): boolean;

    /**
    * @desc: 使用相对route获取绝对route.
    *        curPath= /xxx 或 /xxx/;
                path= aa; 则返回  /xx/aa
                path= ./aa; 则返回  /xx/aa
                path= /aa; 则返回  /aa
    * @return: path.
    */
    function getRoutePath(curPath: string, path: string): string;

    /**
    * @desc: 转义 空格<>&"等.
    * @return: string
    */
    function escapeHtml(str: string): string;

    /**
    * @desc: 获得html字符串
    * @param obj: vue对象, Dom对象 或字符串.
    * @return: 
    */
    function getDomHtmlString(obj: any): string;

    /**
    * @desc: 判断是否是vue对象.
    * @return: 
    */
    function isVueObject(obj: any): boolean;

    /**
    * @desc: 组件是import的, 未创建.
    * @return: 
    */
    function isVueComponent(obj: any): boolean;
  }

  /**
  * @desc 
  *    可在组件释放时自动释放所有的timeout计时器.
  *     - 在created中:             this.timer = new timer();
  *     - 在beforeDestroy中:       this.timer.dispose();
  *     - 在使用计时器的函数中调用:   
                                  var t = this.timer.setTimeout(fn, tm);
                                  this.timer.clearTimeout(t);
                                  this.timer.clearAllTimeout();
  *     - 在使用帧动画的函数中调用:   
                                  var t = this.timer.requestAnimationFrame(fn);
                                  this.timer.cancelAnimationFrame(t);
                                  this.timer.clearAllAnimationFrame();
  *     - 使用Promise方式启动计时器: (会在对象释放前安全停止)
                                  this.timer.sleep(1000)
                                  .then(()=>{
                                  });
  */
  class timer {
    constructor();

    dispose(): void;

    /**
    * @desc: Promise方式计时器.(会在对象释放前安全停止)
    * @return: Promise
    */
    sleep(ms: number): Promise<void>;

    /**
    * @desc: 启动计时器.
    */
    setTimeout(fn: () => void, tm: number): any;

    /**
    * @desc: 清除计时器.
    */
    clearTimeout(t: any): void;

    /**
    * @desc: 启动帧动画.
    */
    requestAnimationFrame(fn: (elapse: number) => void): any;

    /**
    * @desc: 清除帧动画.
    */
    cancelAnimationFrame(t: any): void;

    /**
    * @desc: 清除所有计时器.
    */
    clearAllTimeout(): void;

    /**
    * @desc: 清除所有帧动画.
    */
    clearAllAnimationFrame(): void;

    /**
    * @desc: 清除所有计时器与帧动画.
    */
    clearAll(): void;
  }

  namespace tabbar {

    /**
     * @desc: 设置可见度.
     */
    function setBarHidden(isHidden: boolean): void;
    /**
     * @desc: 是否隐藏.
     */
    function barIsHidden(): boolean;
    /**
     * @desc: 当前的选中的tab索引.
     */
    function getBarCurrentTabIndex(): number;
    /**
     * @desc: 设置当前的选中的tab.
     */
    function setBarCurrentTabIndex(tabIndex: number): void;
    function setBarCurrentTabIndex(tabIndex: number, trigger: boolean): void;
    /**
    * @desc: 获得tabbar改变前的index.
    * @return: number.
    */
    function getBarOldTabIndex(): number;
    /**
     * @desc: 获得当前tabbar的高度.
     */
    function getBarHeight(): number;
  }


  namespace menuList {
    /**
    * @desc: 使用menuList id创建查询参数.
    * @return: string.
    */
    function makeParam(uniqueId: string): string;
  }

  namespace fixedbox {
    /**
    * @desc: 隐藏所有fixedbox.
    */
    function hideAll(): void;
  }

  namespace navbar {

    function _getObject(): any;

    function scrollTo(cfg: {
      behavior: 'instant' | 'smooth' | string,
      left: number,
      top: number
    }): void;

    function scrollOffset(): { left: number, top: number };

    function bindScroll(foo: (event: any) => any): void;

    function unbindScroll(foo: (event: any) => any): void;

    /**
     * @desc: push a page.
     * @param animate: 所有允许的动画, 例如: slide, fade, lift;
     *                 ''   - 无动画.
     *                 null - 默认动画
     * @param nowUseAnimate: 当前是否触发动画.
     */
    function push(cfg: {
      path: string,
      params: any,
      animate: 'slide' | 'fade' | 'lift' | '' | string | null,
      nowUseAnimate: boolean
    } | string): void;

    /**
     * @desc: pop a page.
     * @param nowUseAnimate: 当前是否触发动画.
     */
    function pop(cfg?: {
      nowUseAnimate: boolean
    }): void;
    /**
     * @desc: pop to top-page.
     * @param nowUseAnimate: 当前是否触发动画.
     */
    function popToTop(cfg?: {
      nowUseAnimate: boolean
    }): void;
    /**
     * @desc: pop to to route page.
     * @param nowUseAnimate: 当前是否触发动画.
     */
    function popToRoute(cfg: {
      path: string,
      nowUseAnimate: boolean,
    } | string): boolean;
    /**
     * @desc: replace current route.
     * @param animate: 所有允许的动画, 例如: slide, fade, lift
     *                 ''   - 无动画.
     *                 null - 默认动画
     * @param nowUseAnimate: 当前是否触发动画.
     */
    function replace(cfg: {
      path: string,
      params: any,
      animate: 'slide' | 'fade' | 'lift' | '' | string | null,
      nowUseAnimate: boolean,
    } | string): void;
    /**
     * @desc: refresh current route.
     * @param animate: 所有允许的动画, 例如: slide, fade, lift
     *                 ''   - 无动画.
     *                 null - 默认动画
     */
    function refresh(animate?: 'slide' | 'fade' | 'lift' | '' | string): void;

    /**
     * @desc: 设置可见度.
     */
    function setBarHidden(isHidden: boolean): void;
    /**
     * @desc: 设置可见度 (无论在哪个浏览器中).
     */
    function setBarHiddenForce(isHidden: boolean): void;
    /**
     * @desc: 是否隐藏.
     */
    function barIsHidden(): boolean;
    /**
     * @desc: 设置标题.
     */
    function setBarTitle(title: string): void;
    /**
     * @desc: 获取标题.
     */
    function getBarTitle(): string;
    /**
     * @desc: 获得bar的高度.
     */
    function getBarHeight(): number;
    /**
     * @desc: 设置左右按钮文本的颜色.
     */
    function setBarTintColor(color: string): void;
    /**
     * @desc: 设置标题的颜色.
     */
    function setBarTitleTintColor(color: string): void;
    /**
     * @desc: 设置背景色.
     */
    function setBarBgColor(color: string): void;
    /**
     * @desc: 设置左侧文本.
     */
    function setBarLeftItemText(text: string): void;
    /**
     * @desc: 设置左侧icon.
     * @param icon: 可以为 Dom, string, vue-obj, vue-component. 
     *              - ''  : 不显示图标;
     *              - null: 根据auto来自动判断是否显示返回图标.
     */
    function setBarLeftItemIcon(icon: any): void;
    /**
     * @desc: 设置左侧文本颜色.
     */
    function setBarLeftItemTintColor(color: string): void;
    /**
     * @desc: 设置右侧文本.
     */
    function setBarRightItemText(text: string): void;
    /**
     * @desc: 设置右侧icon.
     * @param icon: 可以为 Dom, string, vue-obj, vue-component.
     */
    function setBarRightItemIcon(icon: any): void;
    /**
     * @desc: 设置右侧文本颜色.
     */
    function setBarRightItemTintColor(color: string): void;
  }

  /**
  * @desc: 获取$store对象.
  */
  function getStore(): any;
  /**
  * @desc: 获取$router对象.
  */
  function getRouter(): VueRouter;
}

export namespace base {
  const badge: any;
  const loadmore: any;
  const swiper: any;
  const swiperPage: any;
  const webView: any;
}

export namespace input {
  const checkbox: any;
  const radioButton: any;
  const searchBar: any;
  const slider: any;
  const switchButton: any;
  const textInput: any;
}

export namespace layout {
  const grid_star: any;
  const grid: any;
  const gridCell: any;

  const navbar: any;
  const navbarItemLeft: any;
  const navbarItemRight: any;
  const navbarView_router: any;
  const navbarViewRouter: any;

  const tabbar: any;
  const tabbarItem: any;
  const tabbarItemIcon: any;
  const tabbarItemTitle: any;
  const tabbarView_router: any;
  const tabbarViewRouter: any;

  const tableView: any;
  const tableViewCell: any;
  const tableViewCellCenter: any;
  const tableViewCellLeft: any;
  const tableViewCellRight: any;

  const menuList: any;
  const menuListItem: any;
  const menuListView_router: any;
  const menuListViewRouter: any;

  const scrollView: any;
  const lazyView: any;
}

export namespace widget {
  const actionSheet: any;
  const popover: any;
  const picker: any;
  const dialogbox: any;
  const dialogboxButton: any;
  const dialogboxButtons: any;
  const dialogboxContent: any;
  const dialogboxTitle: any;
}

export namespace icon {
  const iconBack: any;
  const iconCancel: any;
  const iconDelete: any;
  const iconMenu1: any;
  const iconMenu2: any;
  const iconOk: any;
  const iconPlus: any;
  const iconSetting: any;
  const iconUser: any;
}

export namespace all_components {
  // icons.
  const iconBack: any;
  const iconCancel: any;
  const iconDelete: any;
  const iconMenu1: any;
  const iconMenu2: any;
  const iconOk: any;
  const iconPlus: any;
  const iconSetting: any;
  const iconUser: any;
  // input.
  const checkbox: any;
  const radioButton: any;
  const searchBar: any;
  const slider: any;
  const switchButton: any;
  const textInput: any;
  // layout.
  const grid_star: any;
  const grid: any;
  const gridCell: any;

  const navbar: any;
  const navbarItemLeft: any;
  const navbarItemRight: any;
  const navbarView_router: any;
  const navbarViewRouter: any;

  const tabbar: any;
  const tabbarItem: any;
  const tabbarItemIcon: any;
  const tabbarItemTitle: any;
  const tabbarView_router: any;
  const tabbarViewRouter: any;

  const tableView: any;
  const tableViewCell: any;
  const tableViewCellCenter: any;
  const tableViewCellLeft: any;
  const tableViewCellRight: any;

  const menuList: any;
  const menuListItem: any;
  const menuListView_router: any;
  const menuListViewRouter: any;

  const scrollView: any;
  const lazyView: any;

  // widget.
  const actionSheet: any;
  const popover: any;
  const picker: any;

  // base.
  const badge: any;
  const loadmore: any;
  const swiper: any;
  const swiperPage: any;
  const webView: any;
}

// icons.
export const iconBack: any;
export const iconCancel: any;
export const iconDelete: any;
export const iconMenu1: any;
export const iconMenu2: any;
export const iconOk: any;
export const iconPlus: any;
export const iconSetting: any;
export const iconUser: any;

// input.
export const checkbox: any;
export const radioButton: any;
export const searchBar: any;
export const slider: any;
export const switchButton: any;
export const textInput: any;

// layout.
export const grid_star: any;
export const grid: any;
export const gridCell: any;

export const navbar: any;
export const navbarItemLeft: any;
export const navbarItemRight: any;
export const navbarViewRouter: any;
export const navbarView_router: any;

export const tabbar: any;
export const tabbarItem: any;
export const tabbarItemIcon: any;
export const tabbarItemTitle: any;
export const tabbarViewRouter: any;
export const tabbarView_router: any;
export const tabbarContentAlways: any;

export const tableView: any;
export const tableViewCell: any;
export const tableViewCellCenter: any;
export const tableViewCellLeft: any;
export const tableViewCellRight: any;

export const menuList: any;
export const menuListItem: any;
export const menuListViewRouter: any;
export const menuListView_router: any;

export const scrollView: any;
export const lazyView: any;

// widget.
export const actionSheet: any;
export const popover: any;
export const picker: any;
export const dialogbox: any;
export const dialogboxButton: any;
export const dialogboxButtons: any;
export const dialogboxContent: any;
export const dialogboxTitle: any;
export const fixedbox: any;

// base.
export const badge: any;
export const loadmore: any;
export const swiper: any;
export const swiperPage: any;
export const webView: any;